//
//  cadrePhoto.java
//  detectionContours
//
//  Created by Thomas Jannaud on 22/12/09.
//  Copyright 2009 Jannaud's Technical Company. All rights reserved.
//

import java.awt.Graphics;
import javax.swing.JLabel;
import javax.swing.ImageIcon;
import java.awt.Color;
import java.awt.event.*;
import java.util.LinkedList;
import java.util.Collection;

public class cadrePhoto extends JLabel implements MouseListener, MouseMotionListener {
	private LinkedList<myPoint> selection = new LinkedList<myPoint>();
	private LinkedList<myPoint> last_selection = new LinkedList<myPoint>();
	private myGreyPicture image;
	private showMe show_me;
	private contourDetector detectorOfContours;
	
	public cadrePhoto() {
		super("No picture yet");
		addMouseListener(this);
		addMouseMotionListener(this);
		image = new myGreyPicture();
 	}
	
	public void setPicture(String filename) {
		System.out.println("open " + filename);
		image.initWith(filename);
		publish();
	}
	
	public void publish() {
		if (image.isGood()) {
			setIcon(new ImageIcon(image.getGreyImage()));
			setText("");
		}
		else
			setText("No picture yet");
		repaint();
	}
	
	public void reload() {
		if (image.isGood())
			image.reload();
		publish();
	}
	
	public void stop() {
		if (detectorOfContours != null)
			detectorOfContours.finished();
	}
	
	public void detectContours() {
		stop();
		reload();
		if (image.isGood() && isSelection()) {
			detectorOfContours = new contourDetector(image, show_me, selection);
			show_me = new showMe(this);
			show_me.start();
			detectorOfContours.start();
		}
	}
	
	public void deselect() {
		selection.clear();
		repaint();
	}
		
	/* -----------------------------
	 GRAPHIC INTERFACE
	 ----------------------------- */
	
	public void paint(Graphics g) {
		super.paint(g);
		g.setColor(Color.blue);
		if (isSelection() && image.isGood()) {
				for(myPoint p : selection)
					g.drawRect(p.x, p.y, 0, 0);
				g.drawLine(selection.getLast().x, selection.getLast().y, selection.getFirst().x, selection.getFirst().y);
		}
	}
	
	private boolean isSelection() {
			return !selection.isEmpty();
	}
	
	public void mousePressed(MouseEvent e) {
		if (image.isGood()) {
			last_selection.clear();
			for(myPoint p : selection)
				last_selection.addLast(new myPoint(p));
			selection.clear();
			int x = Math.min(e.getX(), image.getWidth());
			int y = Math.min(Math.max(0, e.getY()), image.getHeight());
			selection.add(new myPoint(x, y, myPoint.CONTOURS));
		}
	}

	public void mouseMoved(MouseEvent e) {}
	
	public void mouseDragged(MouseEvent e) {
		if (image.isGood()) {
			int x = Math.min(e.getX(), image.getWidth());
			int y = Math.min(Math.max(0, e.getY()), image.getHeight());
			myPoint p = selection.removeLast();
			addLine(selection, p, new myPoint(x, y, myPoint.CONTOURS));
			repaint();
		}
	}
	
	public void mouseReleased(MouseEvent e) {
		if (image.isGood()) {
			if (selection.size() < 15) {
				selection.clear();
				for(myPoint p : last_selection)
					selection.addLast(new myPoint(p));
			}
			complete(selection);
			repaint();
		}
	}
	public void mouseClicked(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	
	// ------------------
	// graphic primitives
	// ------------------
	
	private void addLine(Collection<myPoint> c, final myPoint p1, final myPoint p2) {
		// be careful, this adds p1 and p2
		if (Math.abs(p2.x - p1.x) > Math.abs(p2.y - p1.y)) {
			final int inc_x = (p1.x < p2.x) ? 1 : -1;
			final int delta = Math.abs(p2.x - p1.x);
			final int m = p2.y - p1.y;
			final int inc_y = 2 * Math.abs(m);
			final int dy = m > 0 ? 1 : -1;
			int y = p1.y;
			int s0 = -delta;
			final int inc_s0 = -2 * delta;
			for(int i = p1.x; i != p2.x; i += inc_x) {
				c.add(new myPoint(i, y, myPoint.CONTOURS));
				s0 += inc_y;
				if (s0 > 0) {
					y += dy;
					s0 += inc_s0;
				}
			}
		}
		else {
			final int inc_y = (p1.y < p2.y) ? 1 : -1;
			final int delta = Math.abs(p2.y - p1.y);
			final int m = p2.x - p1.x;
			final int inc_x = 2 * Math.abs(m);
			final int dx = m > 0 ? 1 : -1;
			int x = p1.x;
			int s0 = -delta;
			final int inc_s0 = -2 * delta;
			for(int i = p1.y; i != p2.y; i += inc_y) {
				c.add(new myPoint(x, i, myPoint.CONTOURS));
				s0 += inc_x;
				if (s0 > 0) {
					x += dx;
					s0 += inc_s0;
				}
			}
		}
		c.add(p2);
	}
	
	private void complete(LinkedList<myPoint> c) {
		if (c.size() > 2) {
			myPoint p1 = c.removeLast(), p2 = c.removeFirst();
			addLine(c, p1, p2);
		}
		else
			c.clear();
	}
}