//
//  contourDetector.java
//  detectionContours
//
//  Created by Thomas Jannaud on 23/12/09.
//  Copyright 2009 Jannaud's Technical Company. All rights reserved.
//

import java.util.HashSet;
import java.util.LinkedList;
import java.util.HashMap;
import java.util.Collection;

public class contourDetector extends Thread {
	private myGreyPicture image;
	private int width = 0, height = 0, thickness = 3;
	private Collection<myPoint> selection;
	private boolean notFinished = true;
	private showMe publisher;
	
	public contourDetector(myGreyPicture im, showMe shower, Collection<myPoint> obj) {
		image = im;
		this.width = im.getWidth();
		this.height = im.getHeight();
		publisher = shower;
		selection = obj;
	}
	
	public void finished() {
		notFinished = false;
		if (publisher != null)
			publisher.finished();
	}
	
	public void run() {
		notFinished = true;
		detect_minCutContour(selection);
		if (publisher != null)
			publisher.finished();
	}
	
	public void detect_minCutContour(Collection<myPoint> contour) {
		final int border_thickness = thickness;
		int lastSize = contour.size(), lastVal = 100000;
		Collection<myPoint> contours_before = copy(contour);
		Collection<myPoint> contours = copy(contour);
		int step = 0;
		while(notFinished) {
			step++;
			dilate_contours(contours, border_thickness); 
			partition(contours);	
			myGraph g = myGraph.create_graphe(contours, image, width, height);
			pair<Collection<myPoint>, Integer> p = g.minCut();
			if (p.key.size() == 0)
				break;
			contours = p.key;
			if (contours.equals(contours_before))
				break;
			lastVal = p.value;
			lastSize = contours.size();
			image.unsetRed(contours_before);
			contours_before = copy(contours);
			image.setRed(contours_before);
		}
		if (publisher != null)
			publisher.finished();
		return;
	}
	
	private Collection<myPoint> copy(Collection<myPoint> c) {
		LinkedList l = new LinkedList();
		for(myPoint p : c)
			l.add(new myPoint(p));
		return l;
	}
	
	
	// ---------------
	// graph functions
	// ---------------
	
	private void dilate_contours(Collection<myPoint> contours, int thickness) {
		// add a belt to contours. Points had type CONTOURS, they will have type CONTOURS or BELT
		LinkedList< pair<myPoint, Integer> > queue = new LinkedList< pair<myPoint, Integer> >();
		HashMap<myPoint, Integer> done = new HashMap<myPoint, Integer>();
		
		for(myPoint p : contours)
			queue.add(new pair(p, new Integer(thickness)));		
		
		myPoint.setComparator(myPoint.COMPARE_POINT);
		while (!queue.isEmpty()) {
			pair<myPoint, Integer> pp = queue.removeFirst();
			myPoint p = pp.key;
			int count = pp.value;
			done.put(p, new Integer(count));
			if (count > 0) {
				// 8 connexity neighbours
				for(int i = -1; i <= 1; i++)
					for(int j = -1; j <= 1; j++) {
						myPoint q = new myPoint(p.x + i, p.y + j, myPoint.BELT);
						if (!done.containsKey(q) || done.get(q) < (count - 1))
							queue.addLast(new pair(q, count - 1));
					}
			}
		}
		contours.addAll(done.keySet());
	}
	
	
	public static void partition(Collection<myPoint> belt) {
		// Points had type CONTOURS or BELT, they will have type CONTOURS, INSIDE or OUTSIDE
		// find the leftmost point : it is on the outside belt
		int xmin = 10000;
		myPoint pxMin = null;
		for(myPoint p : belt)
			if (p.x < xmin) {
				pxMin = p;
				xmin = p.x;
			}
		pxMin.type = myPoint.OUTSIDE_BELT;
		
		// all the points connected to it in the belt are outside on the outside
		// delimiter : points marked as CONTOURS
		myPoint.setComparator(myPoint.COMPARE_POINT_AND_TYPE);
		LinkedList<myPoint> queue = new LinkedList<myPoint>();
		queue.add(pxMin);
		while (!queue.isEmpty()) {
			myPoint p = queue.removeFirst();
			for(int i = -1; i <= 1; i += 2)
				for(int j = -1; j <= 1; j += 2) {
					int dx = (i + j) / 2, dy = (i - j) / 2;
					myPoint q = new myPoint(p.x + dx, p.y + dy, myPoint.BELT);
					if (belt.contains(q)) {
						belt.remove(q);
						q.type = myPoint.OUTSIDE_BELT;
						belt.add(q);
						queue.addLast(q);
					}
				}
		}
		
		//	other points are on the inside
		for(myPoint p : belt)
			if (p.type == myPoint.BELT)
				p.type = myPoint.INSIDE_BELT;
	}
}
