//
//  cadrePhoto.java
//  detectionContours
//
//  Created by Thomas Jannaud on 22/12/09.
//  Copyright 2009 Jannaud's Technical Company. All rights reserved.
//

import javax.imageio.ImageIO;
import java.io.File;
import java.awt.Color;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.awt.image.ColorModel;
import java.util.Collection;
import java.awt.Toolkit;
import java.net.URL;
import javax.swing.ImageIcon;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.GraphicsConfiguration;
import java.awt.Transparency;
import java.awt.Graphics;

public class myGreyPicture {
	private BufferedImage image_original, image_rendered;
   private String filename;
	private int width, height;
	private ColorModel colorModele;
	private int[] rgbg = new int[4]; // rgb then grey
	private float[] greyPixels_original; // read line by line
	
	public myGreyPicture() {}
	public void setRed(Collection<myPoint> s) {
		for(myPoint p : s)
			if (p.x >= 0 && p.x < width && p.y >= 0 && p.y < height) {
				final int rgbb = toRGB(255, 0, 0);
				image_rendered.setRGB(p.x, p.y, rgbb);
			}
	}
	public void unsetRed(Collection<myPoint> s) {
		for(myPoint p : s)
			if (p.x >= 0 && p.x < width && p.y >= 0 && p.y < height) {
				final int gris = f_to_i(greyPixels_original[p.x + width * p.y]);
				final int rgbb = toRGB(gris, gris, gris);
				image_rendered.setRGB(p.x, p.y, rgbb);
			}
	}
	public Image getGreyImage() {return image_rendered;}
	public int getWidth() {return isGood() ? width : -1;}
	public int getHeight() {return isGood() ? height : -1;}
	
	public void initWith(String filename) {
		if (this.filename == null || !this.filename.equals(filename)) {
			this.filename = filename;
			try {
				if (filename.startsWith("http://"))
					image_original = toBufferedImage(Toolkit.getDefaultToolkit().createImage(new URL(filename)));
				else if (filename.equals("detout.png"))
					image_original = toBufferedImage(Toolkit.getDefaultToolkit().createImage(getClass().getResource(filename)));
				else {
					File filee = new File(filename);
					if (filee.isFile())
						image_original = ImageIO.read(filee);
					else
						throw new Exception("bad file " + filename);
				}
				reload();
			}
			catch (Exception e) {
				System.out.println("error in reload : " + e);
				image_original = null;
			}
		}
	}
	
	public void reload() {
		image_rendered = copy_image(image_original);
		colorModele = image_original.getColorModel();
		width = image_original.getWidth();
		height = image_original.getHeight();
		convertToBlackAndWhite();
	}
	
	public boolean isGood() {
		return (image_original != null);
	}
	
	private void convertToBlackAndWhite() {
		if (greyPixels_original == null || greyPixels_original.length != width * height) {
			greyPixels_original = new float[width * height];
		}
		for(int i = 0; i < width; i++)
			for(int j = 0; j < height; j++) {
				getPixelComponents(i, j);
				int a = colorToGreyRGB();
				greyPixels_original[j * width + i] = i_to_f(rgbg[3]);
				image_rendered.setRGB(i, j, a);
			}
	}
	
	private void getPixelComponents(int x, int y) {
		if (x < 0 || x >= width || y < 0 || y >= height)
			System.out.println("is asking for pixel(" + x + ", " + y + ")");
		Color c = new Color(image_original.getRGB(x, y));
		rgbg[0] = c.getRed();
		rgbg[1] = c.getGreen();
		rgbg[2] = c.getBlue();
	}
	
	private int colorToGreyRGB() {
		int a = (int)(.212671 * (double)(rgbg[0]) + .715160 * (double)(rgbg[1]) + .072169 * (double)(rgbg[2]));
		rgbg[3] = a;
		return toRGB(a, a, a);
	}
	
	private int f_to_i(float x) {
		return Math.min(255, Math.max(0, (int)(255.0f * x))); 
	}
	private float i_to_f(int x) {
		return (float)(x) / 255.0f;
	}
	private int toRGB(int r, int g, int b) { 
		Color c = new Color(r, g, b);
		return c.getRGB();
	}
	
	public float greyComponent(myPoint p) {
		return greyComponent(p.x, p.y);
	}
	
	public float greyComponent(int x, int y) {
		// image is already in black and white
		if (x < 0 || x >= width || y < 0 || y >= height) {
			System.out.println("greycomponent error out of bound : is asking for pixel(" + x + ", " + y + ")");
			return -1.f;
		}
		return greyPixels_original[x + y * width];
	}
	
	
	public static BufferedImage toBufferedImage(Image image) {
		if (image instanceof BufferedImage)
			return (BufferedImage)image;
		else {    
			image = new ImageIcon(image).getImage();
			
			BufferedImage bimage = null;
			GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
			try {
            GraphicsDevice gs = ge.getDefaultScreenDevice();
            GraphicsConfiguration gc = gs.getDefaultConfiguration();
            bimage = gc.createCompatibleImage(image.getWidth(null), image.getHeight(null), Transparency.OPAQUE);
			} catch (Exception e) {
            // The system does not have a screen
			}
			
			if (bimage == null)
            bimage = new BufferedImage(image.getWidth(null), image.getHeight(null), BufferedImage.TYPE_INT_RGB);
			
			// Copy image to buffered image
			Graphics g = bimage.createGraphics();
			
			// Paint the image onto the buffered image
			g.drawImage(image, 0, 0, null);
			g.dispose();
			
			return bimage;
		}
	}
	
	public static BufferedImage copy_image(BufferedImage image) {
		int w = image.getWidth(null), h = image.getHeight(null);
		BufferedImage thecopy = new BufferedImage(w, h, BufferedImage.TYPE_INT_RGB);
		for(int i = 0; i < w; i++)
			for(int j = 0; j < h; j++)
				thecopy.setRGB(i, j, image.getRGB(i, j));
		return thecopy;
	}
}

